import json
import logging
from urllib.parse import urlparse

import jmespath


def load_har(har_path):
    """

    :param har_path:
    :return:
    """
    with open(har_path, 'r', encoding='utf-8') as har:
        return json.load(har)


def save_postman_collection(file_path, postman_json):
    with open(file_path, 'w') as postman:
        postman.write(json.dumps(postman_json, indent=4, ensure_ascii=True))


def extract_params(query):
    """
    :param query: (str) eg: q=testerhome&encoding=utf-8
    :return:
        [
            {
                "key": "q",
                "value": "testerhome"
            },
            {
                 "key": "encoding",
                "value": "utf-8"
            }
        ]
    """
    params = []
    if not query:
        return params

    for i in query.split('&'):
        param = i.split('=')
        key = param[0]
        value = None
        if len(param) > 1:
            value = param[-1]
        params.append({
            'key': key,
            'value': value
        })
    return params


def convert_dict_key(har_dict):
    """

    :param har_dict: [
        {"name": "123", "value": "321"},
        ...
    ]
    :return: [
        {"key": "123", "value": "321"},
        ...
    ]
    """
    for header in har_dict:
        header['key'] = header.pop('name')

    return har_dict


def convert_url(har_request):
    """
    :param har_request: {'url': 'https://www.baidu.com:80/s?wd=12345'}
    :return: {
        'protocol': https,
        'host': ['www', 'baidu', 'com'],
        'port': 80,
        'path': ['s'],
        'query': [
            {'key': 'wd', 'value': '12345'},
            ...
        ]
    }
    """

    url = urlparse(har_request['url'])

    protocol = url.scheme

    host = url.netloc.split('.')

    port = url.port or ""

    path = url.path.split('/')

    # 提取query
    params = extract_params(url.query)

    return {
        'raw': url.path,
        'protocol': protocol,
        'host': host,
        'path': path,
        'port': port,
        'query': params
    }


def convert_headers(har_headers: list):
    """
        :param har_headers: [
            {"name": "123", "value": "321"},
            ...
        ]
        :return: [
            {"key": "123", "value": "321"},
            ...
        ]
        """

    black_list = ['content-length']

    har_headers = list(filter(lambda x: x not in black_list, har_headers))

    def extract(header):
        header['key'] = header.pop('name')
        return header

    return list(map(extract, har_headers))


def convert_body(har_request):
    mime_type = jmespath.search('postData.mimeType', har_request)

    # t=False or t=[None]
    if not mime_type or mime_type is None:
        return None

    if mime_type.find('form-data') != -1:
        return {'mode': 'formdata', 'formdata': convert_dict_key(har_request['postData']['params'])}

    elif mime_type.find('x-www-form-urlencoded') != -1:
        return {'mode': 'urlencoded', 'urlencoded': convert_dict_key(har_request['postData']['params'])}

    elif mime_type.find('json') != -1:
        return {
            'mode': 'raw',
            'raw': har_request['postData']['text']
        }
    elif mime_type.find('plain') != -1:
        return {'mode': mime_type, 'raw': jmespath.search('postData.text', har_request)}

    else:
        logging.error('调用{},参数为：{}'.format(convert_body.__name__, mime_type, har_request))
        raise Exception('无法识别:%s' % mime_type)



import argparse
import sys
import logging
from har2pm.har2postman import Har2Postman
from har2pm import __version__


def main():
    parser = argparse.ArgumentParser(description='har to postman collection')
    parser.add_argument(
        '-V', '--version', dest='version', action='store_true',
        help="show version")
    parser.add_argument(
        '--log-level', default='INFO',
        help="Specify logging level, default is INFO.")
    parser.add_argument(
        'har_path', nargs='?',
        help="har file path")

    args = parser.parse_args()

    log_level = getattr(logging, args.log_level.upper())
    logging.basicConfig(level=log_level)

    if len(sys.argv) == 1:
        # no argument passed
        parser.print_help()
        return 0

    if args.version:
        print("Version:{}".format(__version__))
        return 0

    har_path = args.har_path
    if not har_path or not har_path.endswith('.har'):
        logging.error('HAR file not specified.')
        sys.exit(1)
    har2 = Har2Postman(har_path)
    har2.run()
    return 0



import json
import logging
import os

import jmespath

from har2pm.common import (convert_url, load_har, save_postman_collection, convert_body, convert_headers)


class Har2Postman:

    def __init__(self, har_path):
        self.har_path = har_path
        self.postman_collection = {
            "info": {
                "name": "Har2toPostman",
                "schema": "https://schema.getpostman.com/json/collection/v2.1.0/collection.json"
            },
            'item': []
        }

    def __convert_request(self, request):
        logging.debug('-----------------------item------------------')
        logging.debug('%s' % json.dumps(request, ensure_ascii=False))

        postman_request = {'method': request['method'], 'header': [], 'body': {}, 'url': convert_url(request)}

        if request['method'] == 'POST':
            postman_request['body'] = convert_body(request)

        # 处理 headers
        postman_request['header'] = convert_headers(request['headers'])

        self.postman_collection['item'].append({'name': request['url'], 'request': postman_request})

    def run(self, generate_file=True):
        logging.info(f'read {self.har_path}')

        data = load_har(self.har_path)
        requests = jmespath.search('log.entries[].request', data)

        logging.debug(f'request count: {len(requests)}')

        for request in requests:
            self.__convert_request(request)

        # 是否保存为.json文件
        if generate_file:
            tmp = os.path.splitext(self.har_path)[0]
            postman_collection_path = '{}.json'.format(tmp)
            logging.info('Generate postman collection successfully: %s' % postman_collection_path)
            save_postman_collection(postman_collection_path, self.postman_collection)
        return 0
